@@ -432,6 +432,7 @@ AP_DECLARE(apr_status_t) ap_pcfg_openfile(ap_configfile_t **ret_cfg,
     apr_file_t *file = NULL;
     apr_finfo_t finfo;
     apr_status_t status;
+    int exist_type;
 #ifdef DEBUG
     char buf[120];
 #endif
@@ -457,13 +458,13 @@ AP_DECLARE(apr_status_t) ap_pcfg_openfile(ap_configfile_t **ret_cfg,
     if (status != APR_SUCCESS)
         return status;
 
-    if (finfo.filetype != APR_REG &&
+    exist_type = (finfo.filetype != APR_REG);
 #if defined(WIN32) || defined(OS2) || defined(NETWARE)
-        strcasecmp(apr_filepath_name_get(name), "nul") != 0) {
+    exist_type = (exist_type && strcasecmp(apr_filepath_name_get(name), "nul") != 0);
 #else
-        strcmp(name, "/dev/null") != 0) {
+    exist_type = (exist_type && strcmp(name, "/dev/null") != 0);
 #endif /* WIN32 || OS2 */
-        ap_log_error(APLOG_MARK, APLOG_ERR, 0, NULL,
+    if (exist_type){ ap_log_error(APLOG_MARK, APLOG_ERR, 0, NULL,
                      "Access to file %s denied by server: not a regular file",
                      name);
         apr_file_close(file);
@@ -503,7 +504,7 @@ AP_DECLARE(apr_status_t) ap_pcfg_openfile(ap_configfile_t **ret_cfg,
 #else
     new_cfg->getch = cfg_getch;
     new_cfg->getstr = cfg_getstr;
-    new_cfg->close = cfg_close; 
+    new_cfg->close = cfg_close;
 #endif
     new_cfg->line_number = 0;
     *ret_cfg = new_cfg;
@@ -741,12 +742,12 @@ AP_DECLARE(char *) ap_make_full_path(apr_pool_t *a, const char *src1,
     return path;
 }
 
-static int fname_alphasort(const void *fn1, const void *fn2)
+static int fname_reversealphasort(const void *fn1, const void *fn2)
 {
     const fnames *f1 = fn1;
     const fnames *f2 = fn2;
 
-    return strcmp(f1->fname,f2->fname);
+    return strcmp(f2->fname,f1->fname);
 }
 
 int fnmatch_test(const char *pattern)
@@ -839,7 +840,7 @@ static const char *process_resource_config_nofnmatch(const char *fname,
         apr_dir_close(dirp);
         if (candidates->nelts != 0) {
             qsort((void *) candidates->elts, candidates->nelts,
-                  sizeof(fnames), fname_alphasort);
+                  sizeof(fnames), fname_reversealphasort);
 
             /*
              * Now recurse these... we handle errors and subdirectories
@@ -940,7 +941,7 @@ static const char *process_resource_config_fnmatch(const char *path,
         const char *error;
 
         qsort((void *) candidates->elts, candidates->nelts,
-              sizeof(fnames), fname_alphasort);
+              sizeof(fnames), fname_reversealphasort);
 
         /*
          * Now recurse these... we handle errors and subdirectories
@@ -1027,7 +1028,8 @@ const char *process_command_config(server_rec *s,
 	ap_directive_t *newdir;
 	int optional;
 	char *err = NULL;
-	char *rootpath, *incpath;
+	const char *rootpath, *incpath;
+	char *configfilepath;
 	int li;
 
 	errmsg = populate_include_files(p, ptemp, ari, filename, 0);
@@ -1107,13 +1109,13 @@ ProcessInclude:
 
 				/* we allow APR_SUCCESS and APR_EINCOMPLETE */
 				if (APR_ERELATIVE == status) {
-					rootpath = apr_pstrdup(ptemp, parms->config_file->name);
-					li = strlen(rootpath) - 1;
+					configfilepath = apr_pstrdup(ptemp, parms->config_file->name);
+					li = strlen(configfilepath) - 1;
 
-					while(li >= 0 && rootpath[li] != '/' && rootpath[li] != '\\')
-						rootpath[li--] = 0;
+					while(li >= 0 && configfilepath[li] != '/' && configfilepath[li] != '\\')
+						configfilepath[li--] = 0;
 
-					w = apr_pstrcat(p, rootpath, w, NULL);
+					w = apr_pstrcat(p, configfilepath, w, NULL);
 				}
 				else if (APR_EBADPATH == status) {
 					ap_cfg_closefile(parms->config_file);
@@ -1200,3 +1202,4 @@ Exit:
 
 	return errmsg;
 }
+
